@@ -1,5 +1,5 @@
 # xSGE Physics Framework
-# Copyright (C) 2014-2016 onpon4 <onpon4@riseup.net>
+# Copyright (C) 2014-2017 Julie Marchant <onpon4@riseup.net>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -46,16 +46,16 @@ from __future__ import absolute_import
 from __future__ import print_function
 from __future__ import unicode_literals
 
-__version__ = "0.11.1"
+__version__ = "0.13"
 
 import math
 
 import sge
 
 
-__all__ = ["Collider", "SolidLeft", "SolidRight", "SolidTop", "SolidBottom",
-           "Solid", "SlopeTopLeft", "SlopeTopRight", "SlopeBottomLeft",
-           "SlopeBottomRight", "MobileWall"]
+__all__ = ["Collider", "Wall", "SolidLeft", "SolidRight", "SolidTop",
+           "SolidBottom", "Solid", "SlopeTopLeft", "SlopeTopRight",
+           "SlopeBottomLeft", "SlopeBottomRight", "MobileWall"]
 
 
 NDIG = 6
@@ -84,12 +84,23 @@ class Collider(sge.dsp.Object):
        the wall classes for more information.
 
        Default value: :const:`False`
+
+    .. attribute:: slope_acceleration
+
+       The factor by which the collider accelerates on slopes.  This is
+       multiplied by the :attr:`slope_xacceleration` and
+       :attr:`slope_yacceleration` values of any slopes the collider is
+       touching and added to the use of :attr:`xacceleration` and
+       :attr:`yacceleration` by :meth:`event_update_position`.
+
+       Default value: ``0``
     """
 
     nonstick_left = False
     nonstick_right = False
     nonstick_top = False
     nonstick_bottom = False
+    slope_acceleration = 0
 
     def move_x(self, move, absolute=False, do_events=True, exclude_events=()):
         """
@@ -978,8 +989,49 @@ class Collider(sge.dsp.Object):
         """
         pass
 
+    def event_update_position(self, delta_mult):
+        if delta_mult:
+            xaccel = self.xacceleration
+            yaccel = self.yacceleration
+            if self.slope_acceleration:
+                for slope in set(self.get_left_touching_slope() +
+                                 self.get_right_touching_slope() +
+                                 self.get_top_touching_slope() +
+                                 self.get_bottom_touching_slope()):
+                    xaccel += slope.slope_xacceleration * self.slope_acceleration
+                    yaccel += slope.slope_yacceleration * self.slope_acceleration
 
-class SolidLeft(sge.dsp.Object):
+            vi = self.xvelocity
+            vf = vi + xaccel * delta_mult
+            dc = abs(self.xdeceleration) * delta_mult
+            if abs(vf) > dc:
+                vf -= math.copysign(dc, vf)
+            else:
+                vf = 0
+            self.xvelocity = vf
+            self.move_x(((vi + vf) / 2) * delta_mult)
+
+            vi = self.yvelocity
+            vf = vi + yaccel * delta_mult
+            dc = abs(self.ydeceleration) * delta_mult
+            if abs(vf) > dc:
+                vf -= math.copysign(dc, vf)
+            else:
+                vf = 0
+            self.yvelocity = vf
+            self.move_y(((vi + vf) / 2) * delta_mult)
+
+
+class Wall(sge.dsp.Object):
+
+    """
+    Base class for all wall objects that :class:`Collider` objects
+    interact with in some way.  It is functionally identical to its
+    parent class, :class:`sge.dsp.Object`.
+    """
+
+
+class SolidLeft(Wall):
 
     """
     Class for walls which stop movement of :class:`Collider` objects
@@ -996,7 +1048,7 @@ class SolidLeft(sge.dsp.Object):
         pass
 
 
-class SolidRight(sge.dsp.Object):
+class SolidRight(Wall):
 
     """
     Class for walls which stop movement of :class:`Collider` objects
@@ -1013,7 +1065,7 @@ class SolidRight(sge.dsp.Object):
         pass
 
 
-class SolidTop(sge.dsp.Object):
+class SolidTop(Wall):
 
     """
     Class for walls which stop movement of :class:`Collider` objects
@@ -1030,7 +1082,7 @@ class SolidTop(sge.dsp.Object):
         pass
 
 
-class SolidBottom(sge.dsp.Object):
+class SolidBottom(Wall):
 
     """
     Class for walls which stop movement of :class:`Collider` objects
@@ -1056,7 +1108,37 @@ class Solid(SolidLeft, SolidRight, SolidTop, SolidBottom):
     """
 
 
-class SlopeTopLeft(sge.dsp.Object):
+class Slope(Wall):
+
+    """
+    Base class for all slopes.
+
+    .. attribute:: slope_xacceleration
+
+       Indicates the amount of horizontal acceleration to apply to any
+       collider which is touching the slope.  It is multiplied by the
+       affected colliders' :attr:`slope_acceleration` values and added
+       to their use of :attr:`xacceleration` in
+       :meth:`event_update_position`.
+
+       Default value: ``0``
+
+    .. attribute:: slope_yacceleration
+
+       Indicates the amount of vertical acceleration to apply to any
+       collider which is touching the slope.  It is multiplied by the
+       affected colliders' :attr:`slope_acceleration` values and added
+       to their use of :attr:`yacceleration` in
+       :meth:`event_update_position`.
+
+       Default value: ``0``
+    """
+
+    slope_xacceleration = 0
+    slope_yacceleration = 0
+
+
+class SlopeTopLeft(Slope):
 
     """
     A parent class for slopes which point in some direction upwards and
@@ -1082,6 +1164,14 @@ class SlopeTopLeft(sge.dsp.Object):
        collider's :attr:`nonstick_right` value is :const:`True`.
 
        Default value: :const:`False`
+
+    .. attribute:: slope_xacceleration
+
+       See the documentation for :attr:`Slope.slope_xacceleration`.
+
+    .. attribute:: slope_yacceleration
+
+       See the documentation for :attr:`Slope.slope_yacceleration`.
     """
 
     xsticky_top = False
@@ -1128,7 +1218,7 @@ class SlopeTopLeft(sge.dsp.Object):
         pass
 
 
-class SlopeTopRight(sge.dsp.Object):
+class SlopeTopRight(Slope):
 
     """
     A parent class for slopes which point in some direction upwards and
@@ -1155,6 +1245,14 @@ class SlopeTopRight(sge.dsp.Object):
        :const:`True`.
 
        Default value: :const:`False`
+
+    .. attribute:: slope_xacceleration
+
+       See the documentation for :attr:`Slope.slope_xacceleration`.
+
+    .. attribute:: slope_yacceleration
+
+       See the documentation for :attr:`Slope.slope_yacceleration`.
     """
 
     xsticky_top = False
@@ -1201,7 +1299,7 @@ class SlopeTopRight(sge.dsp.Object):
         pass
 
 
-class SlopeBottomLeft(sge.dsp.Object):
+class SlopeBottomLeft(Slope):
 
     """
     A parent class for slopes which point in some direction upwards and
@@ -1227,6 +1325,14 @@ class SlopeBottomLeft(sge.dsp.Object):
        collider's :attr:`nonstick_right` value is :const:`True`.
 
        Default value: :const:`False`
+
+    .. attribute:: slope_xacceleration
+
+       See the documentation for :attr:`Slope.slope_xacceleration`.
+
+    .. attribute:: slope_yacceleration
+
+       See the documentation for :attr:`Slope.slope_yacceleration`.
     """
 
     xsticky_bottom = False
@@ -1273,7 +1379,7 @@ class SlopeBottomLeft(sge.dsp.Object):
         pass
 
 
-class SlopeBottomRight(sge.dsp.Object):
+class SlopeBottomRight(Slope):
 
     """
     A parent class for slopes which point in some direction upwards and
@@ -1300,6 +1406,14 @@ class SlopeBottomRight(sge.dsp.Object):
        :const:`True`.
 
        Default value: :const:`False`
+
+    .. attribute:: slope_xacceleration
+
+       See the documentation for :attr:`Slope.slope_xacceleration`.
+
+    .. attribute:: slope_yacceleration
+
+       See the documentation for :attr:`Slope.slope_yacceleration`.
     """
 
     xsticky_bottom = False
@@ -1346,7 +1460,7 @@ class SlopeBottomRight(sge.dsp.Object):
         pass
 
 
-class MobileWall(sge.dsp.Object):
+class MobileWall(Wall):
 
     """
     A parent class for walls and slopes that can move.  When an object
